@@ -2,7 +2,8 @@ import numpy as np
 from wotan import flatten
 import tglc
 
-def bilinear(x, y, repeat=23):
+
+def bilinear(x, y, repeat=1):
     '''
     A bilinear formula
     np.array([1 - x - y + x * y, x - x * y, y - x * y, x * y] * repeat)
@@ -86,8 +87,8 @@ def get_psf(source, factor=2, psf_size=11, edge_compression=1e-4, c=np.array([0,
         A[:, -3] = xx.flatten()
     star_info = []
     for i in range(len(source.gaia)):
-    #     if i == 8:
-    #         continue
+        #     if i == 8:
+        #         continue
         x_psf = factor * (x_p[left[i]:right[i]] - x_round[i] + half_size) + (x_shift[i] % 1) // (1 / factor)
         y_psf = factor * (y_p[down[i]:up[i]] - y_round[i] + half_size) + (y_shift[i] % 1) // (1 / factor)
         x_psf, y_psf = np.meshgrid(x_psf, y_psf)  # super slow here
@@ -95,10 +96,11 @@ def get_psf(source, factor=2, psf_size=11, edge_compression=1e-4, c=np.array([0,
         index = coord[down[i]:up[i], left[i]:right[i]]
         A[np.repeat(index, 4), np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F')] += \
             flux_ratio[i] * bilinear(x_residual[i], y_residual[i], repeat=len(a))
+        # star_info.append(
+        #     (np.repeat(index, 4), np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F'),
+        #      flux_ratio[i] * bilinear(x_residual[i], y_residual[i], repeat=len(a))))
         star_info.append(
-            (np.repeat(index, 4), np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F'),
-             flux_ratio[i] * bilinear(x_residual[i], y_residual[i], repeat=len(a))))
-
+            (index, a, flux_ratio[i] * bilinear(x_residual[i], y_residual[i])))
     coord_ = np.arange(- psf_size * factor / 2 + 1, psf_size * factor / 2 + 2)
     x_coord, y_coord = np.meshgrid(coord_, coord_)
     variance = psf_size
@@ -139,7 +141,11 @@ def fit_psf(A, source, over_size, power=0.8, time=0):
     b = b / scaler
     alpha = np.dot(a.T, a)
     beta = np.dot(a.T, b)
-    fit = np.linalg.solve(alpha, beta)
+    try:
+        fit = np.linalg.solve(alpha, beta)
+    except np.linalg.LinAlgError:
+        fit = np.full(np.shape(a)[1], np.nan)
+        # fit = np.linalg.lstsq(a, b, rcond=None)[0]
     return fit
 
 
@@ -168,6 +174,11 @@ def fit_lc(A, source, star_info=None, x=0., y=0., star_num=0, factor=2, psf_size
     whether the star is 2 pixels or closer to the edge of a CCD
     :return: aperture lightcurve, PSF lightcurve, vertical pixel coord, horizontal pixel coord, portion of light in aperture
     """
+    over_size = psf_size * factor + 1
+    a = star_info[star_num][1]
+    star_info_num = (np.repeat(star_info[star_num][0], 4),
+                     np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F'),
+                     np.tile(star_info[star_num][2], len(a)))
     size = source.size  # TODO: must be even?
     # star_position = int(x + source.size * y - 5 * size - 5)
     # aper_lc
@@ -182,8 +193,8 @@ def fit_lc(A, source, star_info=None, x=0., y=0., star_num=0, factor=2, psf_size
     A_cut = np.zeros((len(index), np.shape(A)[1]))
     for i in range(len(index)):
         A_ = np.zeros(np.shape(A)[-1])
-        star_pos = np.where(star_info[star_num][0] == index[i])[0]
-        A_[star_info[star_num][1][star_pos]] = star_info[star_num][2][star_pos]
+        star_pos = np.where(star_info_num[0] == index[i])[0]
+        A_[star_info_num[1][star_pos]] = star_info_num[2][star_pos]
         A_cut[i] = A[index[i], :] - A_
     aperture = np.zeros((len(source.time), len(index)))
     for j in range(len(source.time)):
@@ -214,7 +225,7 @@ def fit_lc(A, source, star_info=None, x=0., y=0., star_num=0, factor=2, psf_size
     else:
         bg_dof = 3
     A = np.zeros((psf_size ** 2, over_size ** 2 + bg_dof))
-    A[np.repeat(index, 4), star_info[star_num][1]] = star_info[star_num][2]
+    A[np.repeat(index, 4), star_info_num[1]] = star_info_num[2]
     psf_shape = np.dot(e_psf, A.T).reshape(len(source.time), psf_size, psf_size)
     psf_sim = psf_shape[:, down_:up_, left_: right_]
     # psf_sim = np.transpose(psf_shape[:, down_:up_, left_: right_], (0, 2, 1))
@@ -251,8 +262,8 @@ def fit_lc(A, source, star_info=None, x=0., y=0., star_num=0, factor=2, psf_size
     return aperture, psf_lc, y - down, x - left, portion
 
 
-def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([]), star_num=0, factor=2, psf_size=11, e_psf=None,
-           near_edge=False, prior=0.001):
+def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([]), star_num=0, factor=2, psf_size=11,
+                       e_psf=None, near_edge=False, prior=0.001):
     """
     Produce matrix for least_square fitting without a certain target
     :param A: np.ndarray, required
@@ -277,6 +288,11 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
     whether the star is 2 pixels or closer to the edge of a CCD
     :return: aperture lightcurve, PSF lightcurve, vertical pixel coord, horizontal pixel coord, portion of light in aperture
     """
+    over_size = psf_size * factor + 1
+    a = star_info[star_num][1]
+    star_info_num = (np.repeat(star_info[star_num][0], 4),
+                     np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F'),
+                     np.tile(star_info[star_num][2], len(a)))
     size = source.size  # TODO: must be even?
     # star_position = int(x + source.size * y - 5 * size - 5)
     # aper_lc
@@ -291,8 +307,8 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
     A_cut = np.zeros((len(index), np.shape(A)[1]))
     for i in range(len(index)):
         A_ = np.zeros(np.shape(A)[-1])
-        star_pos = np.where(star_info[star_num][0] == index[i])[0]
-        A_[star_info[star_num][1][star_pos]] = star_info[star_num][2][star_pos]
+        star_pos = np.where(star_info_num[0] == index[i])[0]
+        A_[star_info_num[1][star_pos]] = star_info_num[2][star_pos]
         A_cut[i] = A[index[i], :] - A_
     aperture = np.zeros((len(source.time), len(index)))
     for j in range(len(source.time)):
@@ -329,6 +345,10 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
     A_[:(cut_size ** 2), -3] = xx.flatten()
     psf_sim = np.zeros((len(source.time), 11 ** 2 + len(field_star_num), len(field_star_num)))
     for j, star in enumerate(field_star_num):
+        a = star_info[star][1]
+        star_info_star = (np.repeat(star_info[star][0], 4),
+                          np.array([a, a + 1, a + over_size, a + over_size + 1]).flatten(order='F'),
+                          np.tile(star_info[star][2], len(a)))
         delta_x = x[star_num] - x[star]
         delta_y = y[star_num] - y[star]
         # for psf_sim
@@ -350,7 +370,7 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
         coord = np.arange(psf_size ** 2).reshape(psf_size, psf_size)
         index = coord[down_11:up_11, left_11:right_11]
         A = np.zeros((psf_size ** 2, over_size ** 2 + bg_dof))
-        A[np.repeat(index, 4), star_info[star][1]] = star_info[star][2]
+        A[np.repeat(index, 4), star_info_star[1]] = star_info_star[2]
         psf_shape = np.dot(e_psf, A.T).reshape(len(source.time), psf_size, psf_size)
         epsf_sum = np.sum(np.nanmedian(psf_shape, axis=0))
         psf_sim_index = coord[down_shift:up_shift, left_shift:right_shift].flatten()
@@ -359,14 +379,8 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
         if star != star_num:
             psf_sim[:, 11 ** 2 + j, j] = np.ones(len(source.time)) / (
                     prior * 1.5e4 * 10 ** ((10 - source.gaia[star]['tess_mag']) / 2.5))
-            # print(np.isnan(psf_sim).any())
         else:
             portion = np.nansum(psf_shape[:, 4:7, 4:7]) / np.nansum(psf_shape)
-    # plt.imshow(np.dot(psf_sim[0, :, :121].T, source.gaia['tess_flux_ratio'][field_star_num]).reshape(11, 11))
-    # plt.show()
-    # print(x[star_num], y[star_num])
-    # plt.imshow(source.flux[0])
-    # plt.show()
 
     star_index = np.where(np.array(field_star_num) == star_num)[0]
     field_star = psf_sim[0, np.arange(11 ** 2).reshape(11, 11)[3:8, 3:8], :].reshape(cut_size ** 2,
@@ -378,7 +392,7 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
             psf_lc[j] = np.nan
         else:
             aper_flat = aperture[j, :, :].flatten()
-            aper_flat = np.append(aper_flat, np.zeros(len(field_star_num) - 1)) #  / prior
+            aper_flat = np.append(aper_flat, np.zeros(len(field_star_num) - 1))  # / prior
             aper_flat[cut_size ** 2 + star_index] = 0
             postcards = psf_sim[j, np.arange(11 ** 2).reshape(11, 11)[3:8, 3:8], :].reshape(cut_size ** 2,
                                                                                             len(field_star_num))
@@ -386,12 +400,10 @@ def fit_lc_float_field(A, source, star_info=None, x=np.array([]), y=np.array([])
             field_star = postcards * source.gaia['tess_flux_ratio'][field_star_num]
             field_star[:, star_index] = 0
             # A_[:(cut_size ** 2), -4] = np.sum(field_star, axis=1)
-            A_[cut_size ** 2:, :len(field_star_num)] = psf_sim[j, 11 ** 2:, :].reshape(len(field_star_num), len(field_star_num))
+            A_[cut_size ** 2:, :len(field_star_num)] = psf_sim[j, 11 ** 2:, :].reshape(len(field_star_num),
+                                                                                       len(field_star_num))
             a = np.delete(A_, cut_size ** 2 + star_index, 0)
             psf_lc[j] = np.linalg.lstsq(a, aper_flat)[0][star_index]
-    # plt.plot(source.time, psf_lc, '.')
-    # plt.show()
-    # print(psf_lc)
     return aperture, psf_lc, y[star_num] - down, x[star_num] - left, portion
 
 
@@ -430,14 +442,26 @@ def bg_mod(source, q=None, aper_lc=None, psf_lc=None, portion=None, star_num=0,
     if np.isnan(local_bg):
         local_bg = 0
     psf_lc = psf_lc - local_bg
-    cal_aper_lc = flatten(source.time, aper_lc / np.nanmedian(aper_lc), window_length=1, method='biweight',
-                          return_trend=False)
+
+    # removes very large outliers to prevent wotan to freeze
+    cal_aper_lc = aper_lc / np.nanmedian(aper_lc)
+    cal_aper_lc[np.where(cal_aper_lc > 100)] = np.nan
+    if np.isnan(cal_aper_lc).all():
+        print('Calibrated aperture flux are not accessible or processed incorrectly. ')
+    else:
+        cal_aper_lc = flatten(source.time, cal_aper_lc, window_length=1, method='biweight',
+                              return_trend=False)
     if near_edge:
         cal_psf_lc = psf_lc
         return local_bg, aper_lc, psf_lc, cal_aper_lc, cal_psf_lc
     else:
-        cal_psf_lc = flatten(source.time, psf_lc / np.nanmedian(psf_lc), window_length=1, method='biweight',
-                             return_trend=False)
+        cal_psf_lc = psf_lc / np.nanmedian(psf_lc)
+        cal_psf_lc[np.where(cal_psf_lc > 100)] = np.nan
+        if np.isnan(cal_psf_lc).all():
+            print('Calibrated PSF flux are not accessible or processed incorrectly. ')
+        else:
+            cal_psf_lc = flatten(source.time, cal_psf_lc, window_length=1, method='biweight',
+                                 return_trend=False)
     # aper_mad = 1.4826 * np.nanmedian(np.abs(cal_aper_lc - 1))
     # if aper_mad > 0.02:
     #     psf_mad = 1.4826 * np.nanmedian(np.abs(cal_psf_lc - 1))
